# -*- coding: utf-8 -*-
"""
Created on Mon Aug 10 23:50:51 2020

@author: Inga Kuznetsova
"""
import numpy as np
from Heteroscedasticity_test_general import Heteroscedasticity_tests
class Glejser_test(Heteroscedasticity_tests):
     '''
     Glejser_test class tests heteroscedasticity computing p value for linear regression between function of feature in original regression under the test and 
     absolute value of residuals

     Attributes:
            numpy array X 
            numpy array Y
     '''
     def __init__(self, X=None, Y=None):
         Heteroscedasticity_tests.__init__(self, X, Y)
         #functions from feature for different linear regressions in Glejser test
         self.features = [np.abs(self.X), np.sqrt(np.abs(self.X)), np.reciprocal(np.abs(self.X))]
         #dictionary for xlabel from test number
         self.var_xlabel={1: '|x|', 2: 'sqrt(|x|)', 3: '1/|x|'}
         
         
     def choose_test(self):
        '''
         Function chooses regression for Glejser test with the best R2 score
         
         args:  none
         returns: maximum R2 score, and number of Glejser regression with this score, p value for this test
        '''
        scores=[]
        for j in range(len(self.features)):
            scores.append(self.find_p_value(self.features[j], self.y_new))
        #array with R2 scores
        R2s=[i[0] for i in scores]
        #array with p values
        pvalues =[i[1] for i in scores]
        #maximum R2 score
        mR2=max(R2s)
        #test wirh mR2
        n=R2s.index(mR2)+1
        del scores
        print("Test number {} works the best with R2={} and p value={}".format(n, mR2, pvalues[n-1]))
        return mR2, n, pvalues[n-1]
    
     def glejser_test(self):
         '''
         Function computes p value for Glejser test regression
         args:  none
         returns: p value to test slope for regression between considered feature and squared residuals. If p < 0.05 we rather have heteroscedasticity.
         '''
         R2, n, pvalue= self.choose_test()
         self.plot_data(self.X, self.Y, 'X', 'Y', title='original data')
        
         if pvalue > 0.00001:
            return "Glejser test: P value = {} is larger than 0.00001, you may not have Heteroscedasticity, check the Glejser test".format(pvalue)
         else:
            return "Glejser test: P value = {} is smaller than 0.00001, you have Heteroscedasticity".format(pvalue)
 
     def plot_test(self, n):
         """
         plots absolute value of residuals from feature for test number (int) n
         """
         self.plot_data(self.features[n-1], self.y_new, self.var_xlabel[n], '|residuals|', title = 'Glejser test  #{}'.format(n))
    
 


# -*- coding: utf-8 -*-
"""
Created on Thu Aug  6 12:38:37 2020

@author: Inga Kuznetsova
"""

import numpy as np
import statsmodels.api as sm
import matplotlib.pyplot as plt
class Heteroscedasticity_tests():
      """ Generic class for Heteroscedasticity tests as
            Park and Glejser methods.
            Both methods use linear regression for functions of tested income feature and residuals as outcome
    
        Attributes:
            numpy arrays X, Y
            if you use pandas dataframe df
            X = df['x'].values.reshape(-1,1)
            Y = df['y'].values.reshape(-1,1)
      """
      def __init__(self, X, Y):
         #incoming feature
         self.X = X
         #outcome
         self.Y = Y
         #residuals
         self.y_new=self.find_residuals()
         
        
      def fit_results(self, x, y):
          '''
          fit linear model 
          args: x, y
          returns: model
          '''
          model=sm.regression.linear_model.OLS(x, y)
          return model.fit()
      
      def find_residuals(self):
          '''
          function computes linear regression residuals
          args:
              regression incoming features: x
              regression outcome: y
          returns: residuals
          '''
         # model = lm.LinearRegression()
         # model.fit(self.X, self.Y)
         # Y_pred = model.predict(self.Y)
          return np.abs(self.fit_results(self.X, self.Y).resid)
      
      def find_p_value(self, x, y):
          '''
          computes p value for regression between x and y
          args: x and y, income and outcome for linear regression
          returns: p value
          '''
        
          results = self.fit_results(x, y)
         
          return results.rsquared, results.pvalues[0]
      
      def plot_data(self, x, y, xlabel, ylabel, title=None):

        """Function to plot original data
        
        Args:
           x and y to plot, xlabel, ylabel, title
        Produces:
            plot
            
        """
        # make the plot
        plt.scatter(x, y)
        plt.ylabel(ylabel)
        plt.xlabel(xlabel)
        plt.title(title)
      
        plt.show()

        
        
        
         
    


# -*- coding: utf-8 -*-
"""
Created on Sat Aug  8 10:44:26 2020

@author: Inga Kuznetsova
"""
import numpy as np
from Heteroscedasticity_test_general import Heteroscedasticity_tests
class Park_test(Heteroscedasticity_tests):
     '''
     Park_test class tests heteroscedasticity computing p value for linear regression between feature in original regression under the test and squared residuals

     Attributes:
            numpy array X
            numpy array Y
     '''
     def __init__(self, X=None, Y=None):
         Heteroscedasticity_tests.__init__(self, X, Y)
         self.x = np.log(np.abs(self.X))
         #normalization helped to do P values more stable
         self.y=np.log(np.square(self.y_new)/(np.sum(np.square(self.y_new))/self.y_new.shape[0]))
         
     def park_test(self):
         '''
         Function computes p value for park test regression
         args:  none
         returns: p value to test slope for regression between considered feature and squared residuals. If p < 0.001 we rather have Heteroscedasticity.
         '''
         #considering logs as suggested
         self.plot_data(self.X, self.Y, 'X', 'Y', title='original data')
         pvalue = self.find_p_value(self.x, self.y)[1]
         if pvalue > 0.001:
            return "Park test: P value = {} is larger than 0.001, you may not have heteroscedasticity, check the Glejser test".format(pvalue)
         else:
            return "Park test: P value = {} is smaller than 0.001, you may have heteroscedasticity".format(pvalue)
        
     def plot_log_residuals(self):
         '''
         plots log(residuals^2) from log(|x|)
         '''
         self.plot_data(self.x, self.y, 'log(|X|)', 'log(residuals^2)', title='Park test')
            
 

        
   


# -*- coding: utf-8 -*-
"""
Created on Sun Aug 16 14:50:37 2020

@author: kuzn137
"""

from setuptools import setup

setup(name = 'heteroscedasticity_tests',
      long_description=""" Tests for heteroscedasticity, Park and Glejser""",
      long_description_content_type='text/markdown',
      version = 1.0,
      description = 'Tests for heteroscedasticity, Park and Glejser',
      packages = ['heteroscedasticity_tests'],
      author = 'Inga Kuznetsova',
      author_email = 'kuzn137@gmail.com'
      )